@@ -2,59 +2,108 @@ import os
 import shutil
 import stat
 
+from trashcli.empty.delete_according_date import ContentReader
+from trashcli.trash import DirReader
+from trashcli.trash_dirs_scanner import TopTrashDirRules
 
-class FileSystemListing:
+
+class FileSystemDirReader(DirReader):
     def entries_if_dir_exists(self, path):
         if os.path.exists(path):
             for entry in os.listdir(path):
                 yield entry
+
     def exists(self, path):
         return os.path.exists(path)
 
-class FileSystemReader(FileSystemListing):
-    def is_sticky_dir(self, path):
+
+class FileReader(TopTrashDirRules.Reader):
+    def exists(self, path):  # type: (str) -> bool
+        return os.path.exists(path)
+
+    def is_sticky_dir(self, path):  # type: (str) -> bool
+        return FileSystemReader.is_sticky_dir(path)
+
+    def is_symlink(self, path):  # type: (str) -> bool
+        return FileSystemReader.is_symlink(path)
+
+
+class TopTrashDirRulesFileSystemReader(TopTrashDirRules.Reader):
+    def exists(self, path):  # type: (str) -> bool
+        return FileSystemReader().exists(path)
+
+    def is_sticky_dir(self, path):  # type: (str) -> bool
+        return FileSystemReader().is_sticky_dir(path)
+
+    def is_symlink(self, path):  # type: (str) -> bool
+        return FileSystemReader().is_symlink(path)
+
+
+class FileSystemReader(FileSystemDirReader):
+
+    @staticmethod
+    def is_sticky_dir(path):  # type: (str) -> bool
         import os
         return os.path.isdir(path) and has_sticky_bit(path)
-    def is_symlink(self, path):
+
+    @staticmethod
+    def is_symlink(path):  # type: (str) -> bool
         return os.path.islink(path)
+
+    @staticmethod
+    def contents_of(path):
+        return read_file(path)
+
+
+class FileSystemContentReader(ContentReader):
     def contents_of(self, path):
         return read_file(path)
 
 
-is_sticky_dir=FileSystemReader().is_sticky_dir
+is_sticky_dir = FileSystemReader().is_sticky_dir
 
 
 class FileRemover:
-    def remove_file(self, path):
+    @staticmethod
+    def remove_file(path):
         try:
-            return os.remove(path)
+            os.remove(path)
         except OSError:
             shutil.rmtree(path)
-    def remove_file_if_exists(self,path):
-        if os.path.exists(path): self.remove_file(path)
 
-def contents_of(path): # TODO remove
+    @classmethod
+    def remove_file_if_exists(cls, path):
+        if os.path.lexists(path): cls.remove_file(path)
+
+
+def contents_of(path):  # TODO remove
     return FileSystemReader().contents_of(path)
-def has_sticky_bit(path): # TODO move to FileSystemReader
+
+
+def has_sticky_bit(path):
     import os
     import stat
     return (os.stat(path).st_mode & stat.S_ISVTX) == stat.S_ISVTX
 
+
 def remove_file(path):
-    if(os.path.lexists(path)):
+    if (os.path.lexists(path)):
         try:
             os.remove(path)
         except:
             return shutil.rmtree(path)
 
-def move(path, dest) :
+
+def move(path, dest):
     return shutil.move(path, str(dest))
 
+
 def list_files_in_dir(path):
     for entry in os.listdir(path):
         result = os.path.join(path, entry)
         yield result
 
+
 def mkdirs(path):
     if os.path.isdir(path):
         return
@@ -71,13 +120,6 @@ def open_for_write_in_exclusive_and_create_mode(path):
     return os.open(path, os.O_WRONLY | os.O_CREAT | os.O_EXCL, 0o600)
 
 
-def ensure_dir(path, mode):
-    if os.path.isdir(path):
-        os.chmod(path, mode)
-        return
-    os.makedirs(path, mode)
-
-
 def read_file(path):
     with open(path) as f:
         return f.read()
