@@ -1,31 +1,46 @@
-# Copyright (C) 2011-2021 Andrea Francia Bereguardo(PV) Italy
+# Copyright (C) 2011-2022 Andrea Francia Bereguardo(PV) Italy
 import argparse
 import os
+import sys
 from pprint import pprint
 
+from trashcli.list_mount_points import os_mount_points
+from trashcli.shell_completion import TRASH_DIRS, add_argument_to
+from trashcli.super_enum import SuperEnum
+
 from . import fstab
 from .fs import FileSystemReader, file_size
-from .fstab import VolumesListing
-from .trash import (version, TrashDirReader, path_of_backup_copy, print_version,
-                    maybe_parse_deletion_date, trash_dir_found,
-                    trash_dir_skipped_because_parent_is_symlink,
-                    trash_dir_skipped_because_parent_not_sticky, UserInfoProvider, DirChecker, AllUsersInfoProvider)
-from .trash import TopTrashDirRules
-from .trash import TrashDirsScanner
-from .trash import ParseError
-from .trash import parse_path
+from .fstab import Volumes, VolumesListing
+from .trash import (
+    AllUsersInfoProvider,
+    DirChecker,
+    ParseError,
+    TrashDirReader,
+    UserInfoProvider,
+    maybe_parse_deletion_date,
+    parse_path,
+    path_of_backup_copy,
+    print_version,
+    version,
+)
+from .trash_dirs_scanner import (
+    TopTrashDirRules,
+    TrashDir,
+    TrashDirsScanner,
+    trash_dir_found,
+    trash_dir_skipped_because_parent_is_symlink,
+    trash_dir_skipped_because_parent_not_sticky,
+)
 
 
 def main():
-    import sys
-    import os
-    from trashcli.list_mount_points import os_mount_points
     ListCmd(
         out=sys.stdout,
         err=sys.stderr,
         environ=os.environ,
-        getuid=os.getuid,
+        uid=os.getuid(),
         volumes_listing=VolumesListing(os_mount_points),
+        volumes=fstab.volumes
     ).run(sys.argv)
 
 
@@ -35,10 +50,11 @@ class ListCmd:
                  err,
                  environ,
                  volumes_listing,
-                 getuid,
+                 uid,
+                 volumes,  # type: Volumes
                  file_reader=FileSystemReader(),
                  version=version,
-                 volume_of=fstab.volume_of):
+                 ):
 
         self.out = out
         self.output = ListCmdOutput(out, err)
@@ -46,11 +62,11 @@ class ListCmd:
         self.version = version
         self.file_reader = file_reader
         self.environ = environ
-        self.uid = getuid()
+        self.uid = uid
         self.volumes_listing = volumes_listing
         self.selector = TrashDirsSelector.make(volumes_listing,
                                                file_reader,
-                                               volume_of)
+                                               volumes)
 
     def run(self, argv):
         parser = Parser(os.path.basename(argv[0]))
@@ -77,6 +93,8 @@ class ListCmd:
                             parsed.all_users,
                             self.environ,
                             self.uid)
+        elif parsed.action == Action.print_python_executable:
+            self.print_python_executable()
         else:
             raise ValueError('Unknown action: ' + parsed.action)
 
@@ -85,7 +103,8 @@ class ListCmd:
         import os
         all = sorted([p for p in psutil.disk_partitions(all=True)],
                      key=lambda p: p.device)
-        physical = sorted([p for p in psutil.disk_partitions()], key=lambda p: p.device)
+        physical = sorted([p for p in psutil.disk_partitions()],
+                          key=lambda p: p.device)
         virtual = [p for p in all if p not in physical]
         print("physical ->")
         pprint(physical)
@@ -109,7 +128,8 @@ class ListCmd:
                 path, volume = args
                 trash_dir = TrashDirReader(self.file_reader)
                 for trash_info in trash_dir.list_trashinfo(path):
-                    self._print_trashinfo(volume, trash_info, extractor, show_files)
+                    self._print_trashinfo(volume, trash_info, extractor,
+                                          show_files)
             elif event == trash_dir_skipped_because_parent_not_sticky:
                 path, = args
                 self.output.top_trashdir_skipped_because_parent_not_sticky(path)
@@ -127,10 +147,9 @@ class ListCmd:
                                           environ,
                                           uid)
         for event, args in trash_dirs:
-            pprint(args)
             if event == trash_dir_found:
                 path, volume = args
-                print("%s, %s" % (path, volume))
+                print("%s" % path)
             elif event == trash_dir_skipped_because_parent_not_sticky:
                 path = args
                 print("parent_not_sticky: %s" % (path))
@@ -138,6 +157,10 @@ class ListCmd:
                 path = args
                 print("parent_is_symlink: %s" % (path))
 
+    def print_python_executable(self):
+        import sys
+        print(sys.executable)
+
     def _print_trashinfo(self, volume, trashinfo_path, extractor, show_files):
         try:
             contents = self.file_reader.contents_of(trashinfo_path)
@@ -149,12 +172,14 @@ class ListCmd:
             except ParseError:
                 self.output.print_parse_path_error(trashinfo_path)
             else:
-                attribute = extractor.extract_attribute(trashinfo_path, contents)
+                attribute = extractor.extract_attribute(trashinfo_path,
+                                                        contents)
                 original_location = os.path.join(volume, relative_location)
 
                 if show_files:
                     original_file = path_of_backup_copy(trashinfo_path)
-                    line = format_line2(attribute, original_location, original_file)
+                    line = format_line2(attribute, original_location,
+                                        original_file)
                 else:
                     line = format_line(attribute, original_location)
                 self.output.println(line)
@@ -199,16 +224,20 @@ def description(program_name, printer):
 
 
 class TrashDirsSelector:
-    def __init__(self, current_user_dirs, all_users_dirs, volume_of):
+    def __init__(self,
+                 current_user_dirs,
+                 all_users_dirs,
+                 volumes  # type: Volumes
+                 ):
         self.current_user_dirs = current_user_dirs
         self.all_users_dirs = all_users_dirs
-        self.volume_of = volume_of
+        self.volumes = volumes
 
     def select(self,
                all_users_flag,
                user_specified_dirs,
                environ,
-               uid):
+               uid): # type (bool, List[str], Dict[str, str], int) -> Iterator[Tuple[str, TrashDir[str, str]]]
         if all_users_flag:
             for dir in self.all_users_dirs.scan_trash_dirs(environ, uid):
                 yield dir
@@ -217,31 +246,27 @@ class TrashDirsSelector:
                 for dir in self.current_user_dirs.scan_trash_dirs(environ, uid):
                     yield dir
             for dir in user_specified_dirs:
-                yield trash_dir_found, (dir, self.volume_of(dir))
+                yield trash_dir_found, (
+                    TrashDir(dir, self.volumes.volume_of(dir)))
 
     @staticmethod
-    def make(volumes_listing, file_reader, volume_of):
+    def make(volumes_listing,
+             reader,  # type: TopTrashDirRules.Reader
+             volumes  # type: Volumes
+             ):
         user_info_provider = UserInfoProvider()
         user_dir_scanner = TrashDirsScanner(user_info_provider,
                                             volumes_listing,
-                                            TopTrashDirRules(file_reader),
+                                            TopTrashDirRules(reader),
                                             DirChecker())
         all_users_info_provider = AllUsersInfoProvider()
         all_users_scanner = TrashDirsScanner(all_users_info_provider,
                                              volumes_listing,
-                                             TopTrashDirRules(file_reader),
+                                             TopTrashDirRules(reader),
                                              DirChecker())
         return TrashDirsSelector(user_dir_scanner,
                                  all_users_scanner,
-                                 volume_of)
-
-
-class SuperEnum(object):
-    class __metaclass__(type):
-        def __iter__(self):
-            for item in self.__dict__:
-                if item == self.__dict__[item]:
-                    yield item
+                                 volumes)
 
 
 class Action(SuperEnum):
@@ -250,6 +275,7 @@ class Action(SuperEnum):
     list_trash = 'list_trash'
     list_volumes = 'list_volumes'
     list_trash_dirs = 'list_trash_dirs'
+    print_python_executable = 'print_python_executable'
 
 
 class Parser:
@@ -257,6 +283,7 @@ class Parser:
         self.parser = argparse.ArgumentParser(prog=prog,
                                               description='List trashed files',
                                               epilog='Report bugs to https://github.com/andreafrancia/trash-cli/issues')
+        add_argument_to(self.parser)
         self.parser.add_argument('--version',
                                  dest='action',
                                  action='store_const',
@@ -282,7 +309,8 @@ class Parser:
                                  action='append',
                                  default=[],
                                  dest='trash_dirs',
-                                 help='specify the trash directory to use')
+                                 help='specify the trash directory to use'
+                                 ).complete = TRASH_DIRS
         self.parser.add_argument('--size',
                                  action='store_const',
                                  default='deletion_date',
@@ -298,6 +326,11 @@ class Parser:
                                  action='store_true',
                                  dest='all_users',
                                  help='list trashcans of all the users')
+        self.parser.add_argument('--python',
+                                 dest='action',
+                                 action='store_const',
+                                 const=Action.print_python_executable,
+                                 help=argparse.SUPPRESS)
 
     def parse_list_args(self, args):
         parsed = self.parser.parse_args(args)
