@@ -1,4 +1,4 @@
-from trashcli.put.my_logger import MyLogger
+from trashcli.put.my_logger import MyLogger, LogData
 from trashcli.put.reporter import TrashPutReporter
 from trashcli.put.trash_result import TrashResult
 from trashcli.put.trasher import Trasher
@@ -17,8 +17,9 @@ class TrashAll:
                   user_trash_dir,
                   mode,
                   forced_volume,
+                  home_fallback,
                   program_name,
-                  verbose,
+                  log_data,  # type: LogData
                   environ,
                   uid):
         result = TrashResult(False)
@@ -28,8 +29,9 @@ class TrashAll:
                                         result,
                                         mode,
                                         forced_volume,
+                                        home_fallback,
                                         program_name,
-                                        verbose,
+                                        log_data,
                                         environ,
                                         uid)
         return result
